#!/bin/bash 
#
#  Hamburg University of Technology
#    Institute of Telematics
# 
#    Created on: Jun 18, 2013
#       Author: homac
#
#
# A small helper script to start programs using mmprof to 
# profile their memory usage.
#
# Customize the LIBDIR path to point to the directory which contains 
# the tool libraries (e.g. libmmstats.so and libmmprof.so).
#
#!/bin/bash




# path to the tool libraries (e.g. libmmstats.so and libmmprof.so).
LIBDIR=.



FUNCTION_PREFIX=

TOOL=$1
shift



KNOWN_TOOLS=(mmstats mmprof)

TOOL_VALID=false
for tool in ${KNOWN_TOOLS[*]} ; do
	if [ "${TOOL}" == "${tool}" ] ; then
		TOOL_VALID=true
		break
	fi
done

if ! $TOOL_VALID ; then
	echo "Unknown tool ${TOOL}!"
	echo "Please make sure you provide a known tool name as the first argument, e.g.:" 1>&2
	echo "  ./mma.sh mmstats <command> <command arguments>" 1>&2
	exit -1
fi


LIBRARY="lib${FUNCTION_PREFIX}${TOOL}.so"

LIBPATH="${LIBDIR}/${LIBRARY}"


if ! [ -r "${LIBPATH}" ] || ! [ -f "${LIBPATH}" ] ; then
	echo "ERROR: Library '${LIBRARY} not found in the configured path '${LIBDIR}'!"   1>&2
	echo "  Please customize the variable LIBDIR in the script mma.sh to point"  1>&2
	echo "  to the directory which contains the tool libraries such as 'libmmstats.so'!" 1>&2
	echo "  If you don't know where to find them you can run the following command:" 1>&2
	echo "" 1>&2
	echo "     sudo updatedb && locate ${LIBRARY}" 1>&2
	echo "" 1>&2
	exit -1
fi







if [ -n "${LD_LIBRARY_PATH}" ] ; then
	LD_LIBRARY_PATH=${LD_LIBRARY_PATH}:${LIBDIR}
else
	LD_LIBRARY_PATH=${LIBDIR}
fi

if [ -n "${LD_PRELOAD}" ] ; then
        LD_PRELOAD=${LD_PRELOAD}:${LIBPATH}
else
	LD_PRELOAD=${LIBPATH}
fi





LD_PRELOAD=${LD_PRELOAD} LD_LIBRARY_PATH=${LD_LIBRARY_PATH} $@
