/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Apr 16, 2013
 *      Author: homac
 */

#ifndef MMA_CONFIG_H_
#define MMA_CONFIG_H_




/* Helper macros to get a string value from a macro! */
#define MMA_STRVAL_DELEGATE(V) #V
#define MMA_STRVAL(V) MMA_STRVAL_DELEGATE(V)

/* Helper macros to concatenate too macro values */
#define __MMA_CONCATENATE_MACROS__(X, Y) __MMA_CONCATENATE_MACROS_HELPER__(X, Y)
#define __MMA_CONCATENATE_MACROS_HELPER__(X,Y) X ## Y



/**
 * Version components.
 */

#define MMA_MAJOR    0
#define MMA_MINOR    2
#define MMA_RELEASE  2


/**
 * Full version
 */
#define MMA_VER      MMA_STRVAL(MMA_MAJOR) "." MMA_STRVAL(MMA_MINOR) "." MMA_STRVAL(MMA_RELEASE)







/**
 * Name of the environment variable to customize the name of the
 * libc library to be loaded and wrapped dynamically.
 */
#define ENV_LIB_NAME "MMA_LIB_NAME"

/**
 * Name of the environment variable to customize the name of the
 * malloc library function to be loaded and wrapped dynamically.
 */
#define ENV_LIB_MALLOC_NAME "MMA_LIB_MALLOC_NAME"

/**
 * Name of the environment variable to customize the name of the
 * calloc library function to be loaded and wrapped dynamically.
 */
#define ENV_LIB_CALLOC_NAME "MMA_LIB_CALLOC_NAME"

/**
 * Name of the environment variable to customize the name of the
 * realloc library function to be loaded and wrapped dynamically.
 */
#define ENV_LIB_REALLOC_NAME "MMA_LIB_REALLOC_NAME"

/**
 * Name of the environment variable to customize the name of the
 * free library function to be loaded and wrapped dynamically.
 */
#define ENV_LIB_FREE_NAME "MMA_LIB_FREE_NAME"

/**
 * Whether offheap memory is shared between processes.
 */
#define ENV_OFFHEAP_SHARED "MMA_OFFHEAP_SHARED"



/**
 * Name of the environment variable to customize the output file name.
 */
#define ENV_OUTPUT_FILE "MMA_OUTPUT_FILE"


/**
 * Name of the environment variable to customize the output.
 *
 * Humanized output is enabled per default. You can set it to zero '0' to disable it.
 * See meaning below.
 *
 * Humanized output means:
 * 1. progress information is printed on stderr (e.g. "mmprof: activated")
 * 2. statistics are printed on stdout in format: "name [unit]: value"
 *
 * Non-Humanized output means:
 * 1. No progress information
 * 2. statistic values are printed on stdout in a single line with spaces as
 *    separators (i.e. "value value value value ..\n"). The order of values
 *    is the same as in humanized output.
 */
#define ENV_OUTPUT_HUMANIZED "MMA_OUTPUT_HUMANIZED"

/**
 * Name of the environment to control the suspended state at startup.
 */
#define ENV_START_SUSPENDED "MMA_START_SUSPENDED"


/**
 * Size of the memory pool for memory allocations in data segment.
 * See offheap.h for details.
 */
#ifndef MEMORY_POOL_SIZE
#  define MEMORY_POOL_SIZE  (1024*1024)
#endif

#ifndef STATS_ADDRMAP_INITIAL_SIZE
#  define STATS_ADDRMAP_INITIAL_SIZE 10240
#endif




/**
 * Default name of the library containing the allocator to be analyzed (e.g. libc).
 */
#ifndef DEFAULT_LIB_NAME
// #  define DEFAULT_LIB_NAME "libc.so.6"
#  define DEFAULT_LIB_NAME "/lib/x86_64-linux-gnu/libc.so.6"
#endif


#ifndef MMA_FUNCTION_PREFIX
#	define MMA_FUNCTION_PREFIX
#endif
/**
 * Default name of the malloc function to be instrumented.
 */
#ifndef DEFAULT_LIB_MALLOC_NAME
#  define DEFAULT_LIB_MALLOC_NAME __MMA_CONCATENATE_MACROS__(MMA_FUNCTION_PREFIX, malloc)
#endif

/**
 * Default name of the calloc function to be instrumented.
 */
#ifndef DEFAULT_LIB_CALLOC_NAME
#  define DEFAULT_LIB_CALLOC_NAME __MMA_CONCATENATE_MACROS__(MMA_FUNCTION_PREFIX, calloc)
#endif

/**
 * Default name of the realloc function to be instrumented.
 */
#ifndef DEFAULT_LIB_REALLOC_NAME
#  define DEFAULT_LIB_REALLOC_NAME __MMA_CONCATENATE_MACROS__(MMA_FUNCTION_PREFIX, realloc)
#endif

/**
 * Default name of the free function to be instrumented.
 */
#ifndef DEFAULT_LIB_FREE_NAME
#  define DEFAULT_LIB_FREE_NAME __MMA_CONCATENATE_MACROS__(MMA_FUNCTION_PREFIX, free)
#endif


/**
 * Whether the offheap dynamic memory management uses shared memory (mmap) or
 * a reserved space in the data segment.
 */
#ifndef DEFAULT_OFFHEAP_SHARED
#  define DEFAULT_OFFHEAP_SHARED 0
#endif


/**
 * Default name of the libc.
 */
#ifndef DEFAULT_OUTPUT_FILE
#  define DEFAULT_OUTPUT_FILE "./mma.log"
#endif

/*
 * Default output configuration.
 */
#ifndef DEFAULT_OUTPUT_HUMANIZED
#  define DEFAULT_OUTPUT_HUMANIZED  1
#endif

/*
 * Default start suspended configuration.
 */
#ifndef DEFAULT_START_SUSPENDED
#  define DEFAULT_START_SUSPENDED  0
#endif

extern int output_humanized;



/* global assert macro */
#define ASSERT(exp) assert(exp)


#endif /* MMA_CONFIG_H_ */
