/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Apr 16, 2013
 *      Author: homac
 */
#include "mmprof.h"

#include <pthread.h>


#include "hiddenmm.h"

lock_t* lock = NULL;


void module_fork_handler_parent (void) {
	lock_acquire(lock);
	log_flush();
}

void module_fork_handler_parent_post (void) {

}

void module_fork_handler_child_post (void) {
	lock_release(lock);
}


/**
 * Function to make sure local data has been initialized
 */
void module_init (void) {
	/* nothing to do */
	mp_set_memory_functions (hidden_malloc, hidden_realloc_mp, hidden_free_mp);
	lock = hidden_calloc(1, CACHE_LINE_SIZE*2);
	if ((size_t)lock & ((size_t)CACHE_LINE_SIZE-1)) {
		lock = (void*)(((size_t)lock+CACHE_LINE_SIZE) & ~((size_t)CACHE_LINE_SIZE-1));
	}
	int ret = pthread_atfork(module_fork_handler_parent, module_fork_handler_parent_post, module_fork_handler_child_post);
	if (ret != 0) log_error("pthread_atfork failed");
	module_tmstmp(); /* mark begin */
}

void module_fini (void) {
	module_tmstmp(); /* mark end */
	lock_acquire(lock);
	log_flush();
	lock_release(lock);
}


void module_suspend(void) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d SUSPENDED\n", clock.tv_sec, clock.tv_nsec, gettid());
	} else {
		log_printf("%ld %ld %d %d %x %x %ld\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_SUSPEND);
	}
	lock_release(lock);
}

void module_resume(void) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d SUSPENDED\n", clock.tv_sec, clock.tv_nsec, gettid());
	} else {
		log_printf("%ld %ld %d %d %x %x %ld\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_RESUME);
	}
	lock_release(lock);
}

/* Inlining wrapper functions to bind module to wrappers */

#include "wrapper.h"
