/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Apr 16, 2013
 *      Author: homac
 *
 *
 * For each function of malloc, calloc, realloc and free there is a
 * corresponding function in this profiling module, which is called
 * as a hook.
 *
 */

#ifndef _MMPROFILE_H_
#define _MMPROFILE_H_


#include <gmp.h>

#include "config.h"
#include "log.h"
#include "mma.h"
#include "offheap.h"
#include "origmm.h"
#include "spinlock.h"

#define AC_MALLOC  hidden_malloc
#define AC_CALLOC  hidden_calloc
#define AC_FREE    hidden_free
#define AC_REALLOC hidden_realloc
#define ASSERT(exp) assert(exp)

#include "khash.h"
#include "perfclk.h"
#include "profile.h"


extern lock_t* lock;



static inline
pid_t gettid () {
	return syscall(SYS_gettid);
}

static inline
void module_tmstmp (void) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d tmstmp\n", clock.tv_sec, clock.tv_nsec, gettid());
	} else {
		log_printf("%ld %ld %d %d\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_TMSTMP);
	}
	lock_release(lock);
}

static inline
void module_malloc (void* ptr, size_t size) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d malloc ptr=0X%x size=%ld\n", clock.tv_sec, clock.tv_nsec, gettid(), ptr, size);
	} else {
		log_printf("%ld %ld %d %d %x %ld\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_MALLOC, ptr, size);
	}
	lock_release(lock);
}

static inline
void module_calloc (void* ptr, size_t num, size_t elem) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d calloc ptr=0X%x num=%ld elem=%ld\n", clock.tv_sec, clock.tv_nsec, gettid(), ptr, num, elem);
	} else {
		log_printf("%ld %ld %d %d %x %ld %ld\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_CALLOC, ptr, num, elem);
	}
	lock_release(lock);
}


static inline
void module_free(void* ptr) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d free ptr=0X%x\n", clock.tv_sec, clock.tv_nsec, gettid(), ptr);
	} else {
		log_printf("%ld %ld %d %d %x\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_FREE, ptr);
	}
	lock_release(lock);
}




static inline
void module_realloc(void* newptr, void* ptr, size_t size) {
	lock_acquire(lock);
	perf_clock_t clock;
	perf_clock(&clock);
	if (output_humanized) {
		log_printf("sec=%ld nsec=%ld tid=%d realloc newptr=0X%x ptr=0X%x size=%ld\n", clock.tv_sec, clock.tv_nsec, gettid(), newptr, ptr, size);
	} else {
		log_printf("%ld %ld %d %d %x %x %ld\n", clock.tv_sec, clock.tv_nsec, gettid(), FC_REALLOC, newptr, ptr, size);
	}
	lock_release(lock);
}





#endif /* _MMPROFILE_H_ */
