/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Apr 16, 2013
 *      Author: homac
 */
#include "mmstats.h"

/**
 * Global statistics
 */


int module_initialized = 0;

statistics_t* stats;

__thread mpz_t mpz_tmp;


void module_fork_handler_parent (void) {
	lock_acquire(&stats->lock);
	log_flush();
}

void module_fork_handler_parent_post (void) {
}

void module_fork_handler_child_post (void) {
	stats->instances++;
	lock_release(&stats->lock);
}


/**
 * Function to make sure local data has been initialized
 */
void module_init (void) {
	if (!module_initialized) {
		pthread_atfork(module_fork_handler_parent, module_fork_handler_parent_post, module_fork_handler_child_post);


		mpz_init(mpz_tmp);

		stats = hidden_calloc(sizeof(statistics_t),1);

		perf_clock_mpz(mpz_tmp);
		stats_init(mpz_tmp);



		module_initialized = 1;
	}
}


void module_fini (void) {
	lock_acquire(&stats->lock);
	stats->instances--;

	if (!stats->instances) {
		perf_clock_mpz(mpz_tmp);
		mpz_sub(mpz_tmp, mpz_tmp, stats->clock_start);

		if (output_humanized) {
			log_printf("Gathered memory statistics:\n");
			log_printf("duration [nsecs]: %Zd\n", mpz_tmp);
			log_printf("load_current [byte]: %ld\n", stats->load_current);
			log_printf("load_max [byte]: %ld\n", stats->load_max);
			log_printf("load_total [byte nsec]: %Zd\n", stats->load_total);
			mpz_div(mpz_tmp, stats->load_total, mpz_tmp);
			log_printf("load_avg [byte]: %Zd\n", mpz_tmp);
		} else {
			log_printf("%Zd %ld %ld %Zd ", mpz_tmp, stats->load_current, stats->load_max, stats->load_total);
			mpz_div(mpz_tmp, stats->load_total, mpz_tmp);
			log_printf(" %Zd\n", mpz_tmp);
		}
	}
	lock_release(&stats->lock);
	log_flush();
}

void module_suspend(void) {
}

void module_resume(void) {
}



/* Inlining wrapper functions to bind module to wrappers */

#include "wrapper.h"
