/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Apr 16, 2013
 *      Author: homac
 *
 *
 * For each function of malloc, calloc, realloc and free there is a
 * corresponding function in this statistics module, which is called
 * as a hook.
 *
 */

#ifndef _MMSTATS_H_
#define _MMSTATS_H_


#include <gmp.h>
#include <pthread.h> /* for fork handlers only */
#include "config.h"
#include "hiddenmm.h"
#include "log.h"
#include "mma.h"
#include "offheap.h"
#include "origmm.h"
#include "spinlock.h"

#define AC_MALLOC  hidden_malloc
#define AC_CALLOC  hidden_calloc
#define AC_FREE    hidden_free
#define AC_REALLOC hidden_realloc

#include "stats.h"

extern int module_initialized;


static inline
void module_malloc (void* ptr, size_t size) {
	if (!module_initialized) return;
	lock_acquire(&stats->lock);
	stats_malloc_nolock(ptr, size);
	lock_release(&stats->lock);
}


#define module_calloc(ptr, num, elem) module_malloc(ptr, num*elem)


static inline
void module_free(void* ptr) {
	if (!module_initialized) return;
	lock_acquire(&stats->lock);
	stats_free_nolock(ptr);

	lock_release(&stats->lock);
}


static inline
void module_realloc(void* newptr, void* ptr, size_t size) {
	if (!module_initialized) return;
	lock_acquire(&stats->lock);
	stats_realloc_nolock(newptr, ptr, size);
	lock_release(&stats->lock);
}



#endif /* _MMSTATS_H_ */
