/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Apr 16, 2013
 *      Author: homac
 *
 *
 *
 * Very simple memory management off the heap.
 *
 * The memory is actually a fixed sized area in the data segment.
 *
 * We need this to satisfy calls to memory management functions
 * during library initialization when we don't have the pointers
 * to the original implementation in the libc available. The only
 * functions which make use of it are currently the dlopen and
 * dlsym functions which need just a few bytes.
 */

#ifndef SRC_OFFHEAP_H_
#define SRC_OFFHEAP_H_

#include <stdlib.h>

#include "config.h"
#include "spinlock.h"


#ifndef OFFHEAP_SIZE
#  define OFFHEAP_SIZE 1024*1024
#endif


#if 0
#	define offheap_lock_t		lock_recursive_t
#	define OFFHEAP_LOCK_INITIALIZER {0}
#	define offheap_lock_acquire lock_recursive_acquire
#	define offheap_lock_release lock_recursive_release
#else
#	define offheap_lock_t		lock_t
#	define OFFHEAP_LOCK_INITIALIZER 0
#	define offheap_lock_acquire lock_acquire
#	define offheap_lock_release lock_release
#endif

typedef unsigned char byte_t;

typedef struct {
	size_t size;
	size_t free; /* just a flag to indicate whether the block is currently allocated by application */
}block_t;


typedef struct {
	size_t size;
	size_t brk;
	offheap_lock_t lock;
	byte_t mem [OFFHEAP_SIZE];

} offheap_t;

extern offheap_t* offheap;
extern int offheap_shared;

#define offheap_member(__ptr__) (((byte_t*)__ptr__ >= offheap->mem && (byte_t*)__ptr__ <  offheap->mem + offheap->brk))

void offheap_init(void);
void offheap_fini(void);

void* offheap_malloc (size_t size);
void offheap_free(void* ptr);
void* offheap_calloc(size_t num, size_t elm);
void* offheap_realloc(void* ptr, size_t size);

static inline
void * offheap_realloc_mp (void *ptr, size_t old_size, size_t new_size) {
	return offheap_realloc(ptr, new_size);
}
static inline
void offheap_free_mp (void *ptr, size_t size) {
	offheap_free(ptr);
}


#endif /* SRC_OFFHEAP_H_ */
