/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: Sep 19, 2013
 *      Author: homac
 */



#include "origmm.h"

#include <dlfcn.h>
#include <stdio.h>
#include <string.h>

#include "log.h"



void* (*orig_malloc) (size_t size) = 0;
void  (*orig_free)   (void* ptr) = 0;
void* (*orig_calloc) (size_t num, size_t elems) = 0;
void* (*orig_realloc)     (void *ptr, size_t size) = 0;


void origmm_init(const char* library_name,
		const char* library_malloc_name,
		const char* library_calloc_name,
		const char* library_realloc_name,
		const char* library_free_name) {
	/**
	 * library handle of the libc for dl.. functions.
	 */
	void* library;

	/*
	 * Dynamically load symbols from original malloc functions
	 */
	library = dlopen(library_name,  RTLD_LAZY);
    if (dlerror() != NULL)  {
    	fprintf(stderr, "Error: can't open library %s", library_name);
    	log_error("dlopen");
    }
	orig_malloc = dlsym(library, library_malloc_name);
    if (dlerror() != NULL)  {
    	dlclose(library);
    	log_error("Error: can't find symbol %s in library %s", library_malloc_name, library_name);
    }
	orig_calloc = dlsym(library, library_calloc_name);
    if (dlerror() != NULL)  {
    	dlclose(library);
    	log_error("Error: can't find symbol %s in library %s", library_calloc_name, library_name);
    }
	orig_realloc = dlsym(library, library_realloc_name);
    if (dlerror() != NULL)  {
    	dlclose(library);
    	log_error("Error: can't find symbol %s in library %s", library_realloc_name, library_name);
    }
	orig_free =dlsym(library, library_free_name);
    if (dlerror() != NULL)  {
    	dlclose(library);
    	log_error("Error: can't find symbol %s in library %s", library_free_name, library_name);
    }

}
