/**
 * Hamburg University of Technology
 * Institute of Telematics
 *
 *  Created on: May 16, 2013
 *      Author: homac
 */

#ifndef SRC_WRAPPER_H_
#define SRC_WRAPPER_H_

#include "mma.h"
#include "module.h"

/*
 * WRAPPER FUNCTIONS
 *
 * This wrapper function implementations are meant to be inlined
 * in the object file of the analysis module.
 *
 *
 * This way the implementation of the modules functions are allowed
 * to be static inline which reduces the overhead.
 *
 */


void* DEFAULT_LIB_MALLOC_NAME (size_t size) {
	if (!(mma_state & MMA_INITIALIZED)) {
		return offheap_malloc(size);
	} else {
		module_ignore++;
		void* ptr = orig_malloc(size);
		module_ignore--;
		if (!(module_ignore | (mma_state & MMA_SUSPENDED))) module_malloc(ptr, size);
		return ptr;
	}
}

void* DEFAULT_LIB_REALLOC_NAME (void *ptr, size_t size) {
	if (!(mma_state & MMA_INITIALIZED)) {
		return offheap_realloc(ptr, size);
	} else {
		module_ignore++;
		void* newptr = orig_realloc(ptr, size);
		module_ignore--;
		if (!(module_ignore | (mma_state & MMA_SUSPENDED))) module_realloc(newptr, ptr, size);
		return newptr;
	}
}

void* DEFAULT_LIB_CALLOC_NAME (size_t num, size_t elem) {
	if (!(mma_state & MMA_INITIALIZED)) {
		return offheap_calloc(num, elem);
	} else {
		module_ignore++;
		void* ptr = orig_calloc(num, elem);
		module_ignore--;
		if (!(module_ignore | (mma_state & MMA_SUSPENDED))) module_calloc(ptr, num, elem);
		return ptr;
	}
}

void DEFAULT_LIB_FREE_NAME (void* ptr) {
	if (!(mma_state & MMA_INITIALIZED) || !ptr || offheap_member(ptr)) {
		offheap_free(ptr);
	} else {
		if (!(module_ignore | (mma_state & MMA_SUSPENDED))) module_free(ptr);
		module_ignore++;
		orig_free(ptr);
		module_ignore--;
	}
}



#endif /* SRC_WRAPPER_H_ */
